from os import path

class ConfigPath(object):
    def __getattr__(self, key):
        return_path = path.join(path.dirname(__file__), key + '.cfg')
        if not path.exists(return_path): return None
        return return_path

config_path = ConfigPath()



#!/usr/bin/env python
import logging
from os import path
from random import choice
from string import letters, digits

from sh import git
from tornado import autoreload
from tornado.ioloop import IOLoop
from tornado.options import define, options, parse_config_file
from tornado.web import Application

from magpie.config import config_path
from magpie.handler import urls

class AttrDict(dict):
    def __getattr__(self, key):
        return self[key]
    def __setattr__(self, key, val):
        self[key] = val

def _rand_str(length=64):
    return ''.join([choice(letters + digits) for i in range(0, length)])

root = path.dirname(__file__)
static_path = path.join(root, 'static')
template_path = path.join(root, 'template')

app_config = dict(static_path=static_path,
                  template_path=template_path,
                  login_url='/login')

define('port', default='8080', type=int)
define('testing', default=False, type=bool)
define('repo', default=None, type=str)
define('username', default=None, type=str)
define('pwdhash', default=None, type=str)
parse_config_file(config_path.web)

if options.testing:
    app_config.update(debug=True)

server = Application(urls, **app_config)
server.settings = AttrDict(server.settings)
server.settings.repo = options.repo
server.settings.username = options.username
server.settings.pwdhash = options.pwdhash
server.settings.session = _rand_str()
server.settings.config_path = config_path

def main():
    server.git = git.bake(_cwd=server.settings.repo)
    server.listen(options.port, 'localhost')
    autoreload.start()
    autoreload.watch(config_path.web)
    IOLoop.instance().start()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
from email import message_from_string
from email.header import decode_header
from imaplib import IMAP4, IMAP4_SSL
from os import makedirs
from os.path import exists, join
import re
from sys import exit

import sh
from tornado.options import define, options, parse_config_file

from magpie.config import config_path

class EmailNotesError(Exception):
    pass

define('imap_server', default=None, type=str)
define('username', default=None, type=str)
define('password', default=None, type=str)
define('folder', default=None, type=str)
define('repo', default=None, type=str)
define('use_ssl', default=True, type=bool)
define('default_notebook', default='', type=str)
parse_config_file(config_path.email_notes)

if options.use_ssl:
    imap = IMAP4_SSL(options.imap_server)
else:
    imap = IMAP4(options.imap_server)
imap.login(options.username, options.password)

result, data = imap.select(options.folder)
if result != 'OK':
    raise EmailNotesError(result)
result, messages = imap.search(None, '(UNSEEN)')
if result != 'OK':
    raise EmailNotesError(result)

if messages[0] == '':
    exit()

# TODO there seems to be a bug where other notes in other notebooks can be deleted
# it might be when a folder is created?
git = sh.git.bake(_cwd=options.repo)
for message_index in messages[0].split(' '):
    junk, data = imap.fetch(message_index, '(RFC822)')
    message = message_from_string(data[0][1])
    subject = decode_header(message.get('Subject'))[0]
    if subject[1] is None:
        subject = subject[0]
    else:
        subject = subject[0].decode(subject[1])
    subject = subject.replace('\r\n','')

    append = False
    if options.folder is None or options.folder.lower() == 'inbox':
        with_notebook = r'^\*Note\*\s(.*)\s@(.*)$'
        without_notebook = r'^\*Note\*\s(.*)$'
    else:
        with_notebook = r'^(.*)\s@(.*)$'
        without_notebook = r'^(.*)$'
    regex = re.search(with_notebook, subject)
    if regex is None:
        regex = re.search(without_notebook, subject)
        notebook_name = options.default_notebook
    else:
        notebook_name = regex.group(2)
        if notebook_name.endswith(' +'):
            notebook_name = notebook_name[:-2]
            append = True
    if regex is None:
        continue

    note_name = regex.group(1)
    if note_name.endswith(' +'):
        note_name = note_name[:-2]
        append = True

    path = join(options.repo, notebook_name)
    if not exists(path):
        makedirs(path)
    path = join(path, note_name)
    if append:
        note_file = open(path, 'a')
    else:
        note_file = open(path, 'w')
    payload = message.get_payload()
    if type(payload) == list:
        text = ''
        for part in payload:
            text += part.get_payload()
    text = text.replace('\r\n', '\n')
    note_file.write(text)
    note_file.close()
    git.add(path)
    # TODO add try/except like web to prevent this from failing if there's no change
    git.commit('-m', 'adding %s from email_notes.py' % note_name)



from os.path import exists, join
from re import search

from magic import Magic
from markdown2 import markdown
from sh import ErrorReturnCode_1
from tornado.web import authenticated

from base import BaseHandler

class NoteHandler(BaseHandler):
    def _delete(self, notebook_name, note_name, confirmed=False):
        path = join(self.settings.repo, notebook_name, note_name)
        dot_path = join(self.settings.repo, notebook_name, '.' + note_name)
        if confirmed:
            self.application.git.rm(path)
            if exists(dot_path):
                self.application.git.rm(dot_path)
            self.application.git.commit('-m', 'removing %s' % path)
            self.redirect('/' + notebook_name)
        else:
            self.render('delete.html', notebook_name=notebook_name,
                        note_name=note_name)

    def _edit(self, notebook_name, note_name, note_contents=None,
              confirmed=False, toggle=False):
        path = join(self.settings.repo, notebook_name, note_name)
        if not confirmed:
            note_contents = open(path).read()
            self.render('note.html', notebook_name=notebook_name,
                        note_name=note_name, note_contents=note_contents,
                        edit=True)
        else:
            if toggle:
                f = open(path)
                tmp = []
                # TODO () in the text seems to break this
                search_string = r'(\[.\])\s(%s)' % note_contents
                for line in f.readlines():
                    regex = search(search_string, line)
                    if regex is not None:
                        old = regex.group(1)
                        if old == '[x]':
                            new = '[ ]'
                        else:
                            new = '[x]'
                        line = "%s %s\n" % (new, regex.group(2))
                    tmp.append(line)
                f.close()
                note_contents = ''.join(tmp)

            f = open(path, 'w')
            f.write(note_contents)
            f.close()

            self.application.git.add(path)
            try:
                if note_contents == '':
                    message = 'creating %s' % path
                else:
                    message = 'updating %s' % path
                self.application.git.commit('-m', message)
            except ErrorReturnCode_1 as e:
                if 'nothing to commit' not in e.message:
                    raise
            self.redirect(note_name)

    def _view_plaintext(self, notebook_name, note_name, highlight=None,
                        dot=False):
        if dot:
            path = join(self.settings.repo, notebook_name, '.' + note_name)
        else:
            path = join(self.settings.repo, notebook_name, note_name)
        note_contents = open(path).read()
        note_contents = markdown(note_contents)
        if highlight is not None:
            note_contents = self._highlight(note_contents, highlight)
        self.render('note.html', notebook_name=notebook_name,
                    note_name=note_name, note_contents=note_contents,
                    edit=False, dot=dot)

    def _view_file(self, notebook_name, note_name):
        path = join(self.settings.repo, notebook_name, note_name)
        with open(path, 'rb') as f:
            self.set_header("Content-Disposition", "attachment; filename=%s" % \
                            note_name)
            self.write(f.read())

    @authenticated
    def get(self, notebook_name, note_name):
        notebook_name = notebook_name.replace('+', ' ')
        note_name = note_name.replace('+', ' ')
        action = self.get_argument('a', 'view')
        if action == 'delete':
            self._delete(notebook_name, note_name, confirmed=False)
        elif action == 'edit':
            self._edit(notebook_name, note_name, confirmed=False)
        else:
            path = join(self.settings.repo, notebook_name, note_name)
            dot_path = join(self.settings.repo, notebook_name, '.' + note_name)
            highlight = self.get_argument('hl', None)
            with Magic() as m:
                mime = m.id_filename(path)
                if 'text' in mime or 'empty' in mime:
                    self._view_plaintext(notebook_name=notebook_name,
                                         note_name=note_name,
                                         highlight=highlight)
                elif exists(dot_path):
                    download = self.get_argument('dl', False)
                    if download:
                        self._view_file(notebook_name=notebook_name,
                                        note_name=note_name)
                    else:
                        self._view_plaintext(notebook_name=notebook_name,
                                             note_name=note_name,
                                             highlight=highlight, dot=True)

                else:
                    self._view_file(notebook_name=notebook_name,
                                    note_name=note_name)

    @authenticated
    def post(self, notebook_name, note_name):
        notebook_name = notebook_name.replace('+', ' ')
        note_name = note_name.replace('+', ' ')
        action = self.get_argument('a', 'view')
        if bool(self.get_argument('save', False)):
            note = self.get_argument('note')
            toggle = self.get_argument('toggle', False)
            self._edit(notebook_name=notebook_name, note_name=note_name,
                       note_contents=note, confirmed=True, toggle=toggle)
        elif bool(self.get_argument('delete', False)):
            self._delete(notebook_name, note_name, confirmed=True)
        else:
            self.redirect(note_name)



from urllib2 import quote, unquote

import bcrypt

from base import BaseHandler

class LoginHandler(BaseHandler):
    def get(self):
        error = self.get_argument('error', None)
        self.render('login.html', error=error, hide_notebooks=True)

    def post(self):
        username = self.get_argument('username', '')
        password = self.get_argument('password', '')
        if username == '' or password == '':
            self.redirect('/login?error=' + quote(("Username and password are "
                                                   "required!")))
        elif username != self.settings.username or \
        bcrypt.hashpw(password, self.settings.pwdhash) != self.settings.pwdhash:
            self.redirect('/login?error=' + quote("Username/password "
                                                  "incorrect!"))
        else:
            self.set_cookie('session', self.settings.session)
            self.redirect(self.get_argument('next', '/'))
